﻿// ModuleTextProcessor.as

// Данные редактора для текстового процессора модулей. Пока хранит только флаг нужности показа списка
class ModuleEditorData : EditorData
{
    bool needQuickActivate;
    ModuleEditorData()
    {
        needQuickActivate = false;
    }
};

// Контексты срабатывания подсказки снегопата
enum ActivateModes {
    modeNone,       // нет контекста
    modeQuote,      // внутри кавычек
    modeName,       // обычный режим, в идентификаторе
    modePreproc     // в инструкции препроцессора
};

string getTextLine(TextManager@ tm, uint line)
{
    v8string wline;
    tm.getLine(line, wline);
    return wline.str;
}

bool is_first_name_symbol(wchar_t symbol)
{
    return is_alpha(symbol) || '_' == symbol;
}

bool is_name_symbol(wchar_t symbol)
{
    return is_alnum(symbol) || '_' == symbol;
}

// Выяснить контекст вызова подсказки
// также возвращает начало текущей строки до каретки и последнюю лексему в ней.
ActivateModes getActivateMode(TextManager@ tm, const TextPosition&in caretPos, string&out beginOfLine, string&out lastLexem)
{
    beginOfLine = getTextLine(tm, caretPos.line).rtrim("\r\n");
    int spaces = caretPos.col - beginOfLine.length - 1;
    if (spaces > 0)
        beginOfLine.padRight(' ', spaces);
    else if (spaces < 0)
        beginOfLine.setLength(caretPos.col - 1);

    lex_provider source(beginOfLine.cstr);
    lexem lex;
    for (;;) {
        source.next(lex);
        if (lex.isEmpty() || source.atEnd())
            break;
    }
    if (lexPreproc == lex.type)
        return modePreproc;
    else if (lexRemark == lex.type || lexNumber == lex.type || lexDateOpen == lex.type || lexDate == lex.type)
        return modeNone;
    else if (lexQuoteOpen == lex.type)// Внутри строки
        return modeQuote;
    else {
        if (lexName == lex.type || lexDirective == lex.type)
            lastLexem = lex.text;
        return modeName;
    }
}

class ModuleTextProcessor : TextProcessor
{
    void connectEditor(TextWnd@ editor) { @editor.editorData = ModuleEditorData(); }
    void afterChar(TextWnd@ editor, uint16 symbol)
    {
        //string t = " "; t[0] = symb; Message("WM_CHAR " + uint(symb) + " = " + t);
        if (editor !is activeTextWnd)
            return;		// Не в фокусе не работаем, возникает при наборе в штатной подсказке

        bool bItsNameSymbol = is_name_symbol(symbol);
        ModuleEditorData@ pData = cast<ModuleEditorData>(editor.editorData);
        if (!bItsNameSymbol)
            pData.needQuickActivate = true;
        ITextEditor@ pTxtEdt = editor.ted;
        TextPosition caretPos;
        pTxtEdt.getCaretPosition(caretPos, false);
        if ('\r' == symbol) {
            smartEnter(caretPos, editor);
            return;
        }
        string lineBegin, lastLexem;
        ActivateModes mode = getActivateMode(editor.textDoc.tm, caretPos, lineBegin, lastLexem);
        if (modeName == mode && doAutoReplace(symbol, lineBegin, caretPos, editor))
            return;
        /*
        IProfile* pProf = get_process_service<IProfile>();
        int qaChars;
        if (!pProf.getValue(SETTINGS_QACHARS, qaChars))
            qaChars = 1;
        */
        uint qaChars = 2;

        if (modeName == mode && ((bItsNameSymbol && pData.needQuickActivate && lastLexem.length >= qaChars))) {
            activateInModule(editor, lastLexem, caretPos, mode, symbol);
        }/* else if (modeQuote == mode)
            activateInString(pEditor, caretPos);
        else if (modePreproc == mode)
            activateInPreproc(pEditor, caretPos);
        else if ('(' == symbol || ',' == symbol)
            ParamShow::get().activate();
        */
    }
    void onKeyDown(TextWnd@ editor, uint wParam, uint lParam)
    {
        switch (wParam) {
        case VK_LEFT:
        case VK_RIGHT:
        case VK_UP:
        case VK_DOWN:
        case VK_PRIOR:
        case VK_NEXT:
        case VK_HOME:
        case VK_END:
        case VK_RETURN:
        case VK_BACK:
            cast<ModuleEditorData>(editor.editorData).needQuickActivate = true;
            break;
        case VK_DELETE:
            //m_pModElements.setNeedReparse();
            break;
        case VK_ESCAPE:
            //ParamShow::get().hide();
            break;
        }
    }
    void smartEnter(TextPosition& caretPos, TextWnd@ editor)
    {
        v8string line;
        editor.textDoc.tm.getLine(caretPos.line - 1, line);
        if (isLineEndWithOpenQuote(line.str)) {
            // Заканчивается открытым литералом
            editor.textDoc.tm.getLine(caretPos.line, line);
            if (line.str.ltrim().substr(0, 1) != "|") {
                // Надо вставить |
                insertInSelection(editor.ted, editor.textDoc.tm, editor.textDoc.itm, "|", true, false);
                // Проверял вставку многострока с выравниванием и установкой каретки
                //insertInSelection(editor.ted, editor.textDoc.tm, editor.textDoc.itm, "Если ¦ Тогда\nКонецЕсли;", true, true);
            } else {
                caretPos.col++;
                editor.ted.setCaretPosition(caretPos, false);
            }
        }
    }
    bool doAutoReplace(wchar_t symbol, const string&in lineBegin, TextPosition&in caretPos, TextWnd@ pTxtWnd)
    {
        //if (!bAutoReplaces)
        //    return FALSE;
        //	Делаем автозамены: ? ++ -- += -= *= /= %= 
        uint len = lineBegin.length;
        /*
        if ('?' == symbol) {
            IntelliSite& is = IntelliSite::get();
            is.addItem(Delimeters::getDelimeter(Delimeters::question));
            is.show(CString(), pTxtWnd);
            return TRUE;
        }*/
        if (len < 3)
            return false;
        wchar_t prevSymbol = lineBegin[len - 2];
        string replSymbol1 = "", replSymbol2 = "";
        if ('+' == symbol && '+' == prevSymbol) {
            replSymbol1 = "+";
            replSymbol2 = "1";
        } else if ('-' == symbol && '-' == prevSymbol) {
            replSymbol1 = "-";
            replSymbol2 = "1";
        } else if ('=' == symbol) {
            replSymbol2.insert(0, symbCaret);
            if ('+' == prevSymbol)
                replSymbol1 = "+";
            else if ('-' == prevSymbol)
                replSymbol1 = "-";
            else if ('*' == prevSymbol)
                replSymbol1 = "*";
            else if ('/' == prevSymbol)
                replSymbol1 = "/";
            else if ('%' == prevSymbol)
                replSymbol1 = "%";
        }
        if (replSymbol1.isEmpty())
            return false;
        string spaceSymbol = "";
        int endOfName = len - 3;
        while (endOfName >= 0 && is_space(lineBegin[endOfName])) {
            endOfName--;
            spaceSymbol = " ";
        }
        if (endOfName < 0)
            return false;
        int startOfName = endOfName;
        for (;;) {
            if (!is_space(lineBegin[startOfName])) {
                startOfName--;
                if (startOfName < 0)
                    break;
            } else
                break;
        }
        startOfName++;
        if (startOfName > endOfName || !is_first_name_symbol(lineBegin[startOfName]))
            return false;
        string id = lineBegin.substr(startOfName, endOfName - startOfName + 1);
        string replace = "=" + spaceSymbol + id + spaceSymbol + replSymbol1 + spaceSymbol + replSymbol2;
        string endOfLine = getTextLine(pTxtWnd.textDoc.tm, caretPos.line).substr(caretPos.col - 1).rtrim("\r\n").replace(indentRex, "");
        if (endOfLine.isEmpty())
            replace += ";";

        TextPosition tpStart(caretPos);
        tpStart.col -= 2;
        ITextEditor@ pEditor = pTxtWnd.ted;
        pEditor.setSelection(tpStart, caretPos, false, false);
        insertInSelection(pEditor, pTxtWnd.textDoc.tm, pTxtWnd.textDoc.itm, replace, true, false);
        return true;
    }
    void activateInModule(TextWnd@ editor, const string&in lastLexem, TextPosition& caretPos, ActivateModes mode, wchar_t symbol)
    {
        // Пока только так
        showAssist();
    }
};

void showProfileFolder(IProfileFolder@ folder, const string& level)
{
    string indent = level;
    Message(indent + "[" + folder.getName() + "]");
    indent += "\t";
    Value val;
    v8string vname, vval;
    for (uint k = 0, m = folder.valuesCount(); k < m ; k++) {
        folder.getValueKeyAt(k, vname);
        folder.getValueAt(k, val);
        val.pValue.getString(vval);
        Message(indent + vname + " = " + vval);
    }
    for (uint k = 0, m = folder.foldersCount(); k < m; k++) {
        IProfileFolder@ subFolder;
        folder.getFolderAt(subFolder, k);
        showProfileFolder(subFolder, indent);
    }
}

void showAssist()
{
    //showProfileFolder(getProfileRoot(), "");
    ICommandReceiver@ pCmdRcv = cast<IUnknown>(coreMainFrame);
    if (pCmdRcv !is null) {
        CommandID cmd;
        cmd.group = cmdFrameGroup;
        cmd.num = cmdFrameShowAssist;
        if (pCmdRcv.hasHandler(cmd))
            pCmdRcv.transmitCommand(Command(cmd, 0));
    }
}
