﻿// Snegopat Engine main file
/*
   Основная точка входа. Вызывается в момент, когда snegopat.dll средствами WinAPI
   отловил создание окна заставки 1С, но она еще не показана. Все 1С-ные дллки загружены.
   однако часть функций вызывать еще небезопасно, так как часть инициализации 1С происходит
   после создания основного окна. Поэтому сейчас надо инициализировать то, что нужно и можно
   до создания основного окна, и отлавливать его создание, дабы запустить всё остальное
*/

void main()
{
#if test = 1
        dumpVtable(currentProcess());
#endif
    initSymbolClassTable();
    initSnegopatProfile();
    setTextHooks();
    initMainFrameWork();
}

// Заполняем таблицу, в которой указан класс символа. Таблица используется в snegopat.dll, но
// для заполнения используются функции V*.
void initSymbolClassTable()
{
    for (uint k = 0; k <= 32; k++)
        mem::byte[symbolClassTable + k] = symbSpace;
    for (uint k = 33; k < 0x10000; k++) {
        if (is_space(k))
            mem::byte[symbolClassTable + k] = symbSpace;
        else if (is_alpha(k))
            mem::byte[symbolClassTable + k] = symbAlpha;
    }
}

// Подключение к дереву настроек файла snegopat.pfl
// Для обеспечения портабельности файл лежит в каталоге снегопата, поэтому
// этот каталог должен быть доступен для записи.
void initSnegopatProfile()
{
    IProfileSource@ snegopatProfileSource;
    currentProcess().createByClsid(CLSID_FileProfileSrc, IID_IProfileSource, snegopatProfileSource);
#if test = 1
    dumpVtable(@snegopatProfileSource);
    dumpVtable(getProfileService());
    dumpVtable(getProfileRoot());
#endif
    snegopatProfileSource.init("file://" + myFolder + "snegopat.pfl");
    getProfileService().attachSource(snegopatProfileSource, pflSnegopat);
}

// Установка различных перехватов, связанных с текстовыми окнами/документами
void setTextHooks()
{

    //debugger();
    // Надо установить перехват на создании окон Text control и text view.
    // Объект TextCtrl можно создать прямо сейчас и установить перехват в его виртуальной таблице
    IWindowView@ wnd;
    currentProcess().createByClsid(CLSID_TxtEdtCtrl, IID_IWindowView, wnd);
#if test = 1
    dumpVtable(@wnd, "_TxtEdtCtrl");
#endif
    if (wnd is null)
        Print("Не удалось создать TxtEdtCtrl IWindowView");
    else {
        trTxtEdtCtrl_createWnd.setTrap(wnd, IWindowView_createWindow, TxtEdtCtrl_createWindow);
        ICommandTarget@ tgt = cast<IUnknown>(wnd);
#if test = 1
        dumpVtable(@tgt, "_TxtEdtCtrl");
#endif
        if (tgt is null)
            Print("Не удалось получить ICommandTarget из TxtEdtCtrl IWindowView");
        else // Ставим перехват на получение окном текстового контрола команды
            trTxtCtrCommand_onExecute.setTrap(tgt, ICommandTarget_onExecute, TxtCtrCommand_onExecute);
    }
    // Теперь установим перехват на назначение текстового расширения текстовому документу
    IDocument@ doc;
    currentProcess().createByClsid(CLSID_TxtEdtDoc, IID_IDocument, doc);
#if test = 1
    dumpVtable(@doc, "_TxtEdtDoc");
#endif
    if (doc is null)
        Print("Не удалось создать TxtEdtDoc IDocument");
    else {
        ITextManager_Operations@ to = cast<IUnknown>(doc);
#if test = 1
        dumpVtable(@to, "_TxtEdtDoc");
#endif
        if (to is null)
            Print("Не удалось получить ITextManager_Operations из TxtEdtDoc");
        else
            trTxtMgr_setExtender.setTrap(to, ITextManager_Operations_setExtenderCLSID, TxtMgrOper_setExtender);
        // Далее устанавливаем перехват на TxtEdtDoc::createView
        // он нужен только один раз, чтобы получить готовый объект TxtEdtView, т.к. создать его сейчас
        // самостоятельно ещё рано, программа вылетит.
        trTxtEdtView_createWnd.setTrap(doc, IDocument_createView, TxtDoc_createView);
    }

}

uint TxtDoc_createView(IDocument& doc, uint pView)
{
#if test = 1
    dumpVtable(@doc, "_TxtDoc_createView");
#endif

    trTxtEdtView_createWnd.swap();
    IWindowView@ wnd;
    doc.createView(wnd);

#if test = 1
    dumpVtable(@wnd, "_TxtEdtView");
#endif

    if (wnd is null)
        Print("Не удалось создать TxtEdtView IWindowView");
    else {
        trTxtEdtView_createWnd.setTrap(wnd, IWindowView_createWindow, TxtEdtView_createWindow);
        ICommandTarget@ tgt = cast<IUnknown>(wnd);
#if test = 1
        dumpVtable(@tgt, "_TxtEdtView");
#endif
        if (tgt is null)
            Print("Не удалось получить ICommandTarget из TxtEdtWnd IWindowView");
        else // Ставим перехват на получение окном текстового контрола команды
            trTxtEdtCommand_onExecute.setTrap(tgt, ICommandTarget_onExecute, TxtEdtCommand_onExecute);
    }
    wnd.AddRef();
    mem::dword[pView] = wnd.this();
    return pView;
}

TrapVirtualStdCall trTxtEdtView_createWnd;
TrapVirtualStdCall trTxtEdtCommand_onExecute;
TrapVirtualStdCall trTxtEdtCtrl_createWnd;
TrapVirtualStdCall trTxtCtrCommand_onExecute;
TrapVirtualStdCall trTxtMgr_setExtender;

bool TxtEdtCtrl_createWindow(IWindowView& view, HWND parentWindow)
{

#if test = 1
    dumpVtable(@view, "_TxtEdtCtrl_createWindow");
#endif

    trTxtEdtCtrl_createWnd.swap();
    bool res = view.createWindow(parentWindow);
    trTxtEdtCtrl_createWnd.swap();
    onCreateTextWnd(view, true);

    return res;
}

bool TxtEdtView_createWindow(IWindowView& view, HWND parentWindow)
{

#if test = 1
    dumpVtable(@view, "_TxtEdtView_createWindow");
#endif

    trTxtEdtView_createWnd.swap();
    bool res = view.createWindow(parentWindow);
    trTxtEdtView_createWnd.swap();
    onCreateTextWnd(view, false);

    return res;
}

bool checkCommandAndHideSmartBox(ICommandTarget& tgt, const Command& command, TrapVirtualStdCall& trap)
{
    //Message("Txt View Command execute " + command.id.group + " num=" + command.id.num + " param=" + command.param);
    /*
    if (IntelliSite::get().active()) {
    unsigned int cs = commandState(command.id);
    if (cs & cmdStateEnabled)
    IntelliSite::get().hide();
    }*/
    trap.swap();
    bool res = tgt.onExecute(command);
    trap.swap();
    return res;
}

bool TxtEdtCommand_onExecute(ICommandTarget& tgt, const Command& command)
{

#if test = 1
    dumpVtable(@tgt, "_TxtEdtCommand_onExecute");
#endif
    bool res = checkCommandAndHideSmartBox(tgt, command, trTxtEdtCommand_onExecute);

    return res;
}

bool TxtCtrCommand_onExecute(ICommandTarget& tgt, const Command& command)
{
#if test = 1
    dumpVtable(@tgt, "_TxtCtrCommand_onExecute");
#endif
    bool res = checkCommandAndHideSmartBox(tgt, command, trTxtCtrCommand_onExecute);

    return res;
}

int TxtMgrOper_setExtender(ITextManager_Operations& to, const Guid& clsid)
{
#if test = 1
    dumpVtable(@to, "_TxtMgrOper_setExtender");
#endif
    trTxtMgr_setExtender.swap();
    int res = to.setExtenderCLSID(clsid);
    trTxtMgr_setExtender.swap();
    ITextManager@ itm = cast<IUnknown>(to);
    changeTextExtender(itm, clsid);
    return res;
}
